from __future__ import annotations

import functools
import inspect
from types import ModuleType
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    List,
    Optional,
    Type,
    TypedDict,
    TypeVar,
    Union,
    cast,
)

from importlib_metadata import distribution
from typing_extensions import NotRequired

from .cube import Cube
from .context import channel_instance
from .schema import BaseSchema


class ChannelMeta(TypedDict):
    author: List[str]
    name: NotRequired[str]
    version: NotRequired[str]
    license: NotRequired[str]
    urls: NotRequired[Dict[str, str]]
    description: NotRequired[str]
    icon: NotRequired[str]
    classifier: List[str]
    dependencies: List[str]

    standards: List[str]
    frameworks: List[str]
    config_endpoints: List[str]
    component_endpoints: List[str]


def _default_channel_meta() -> ChannelMeta:
    return ChannelMeta(
        author=[],
        classifier=[],
        dependencies=[],
        standards=[],
        frameworks=[],
        config_endpoints=[],
        component_endpoints=[],
    )


def get_channel_meta(module: str) -> ChannelMeta:
    dist = distribution(module)
    meta = cast(Dict[str, Any], _default_channel_meta())
    meta |= dist.metadata.json
    if meta["author"]:  # "author" in dist.metadata.json and dist.metadata.json["author"]
        meta["author"] = meta["author"].split(",")
    elif "author_email" in meta:
        meta["author"] = meta["author_email"].split(",")
    meta["urls"] = dict(i.split(", ") for i in meta.get("project_url", ()))
    meta["dependencies"] = dist.requires or []
    return cast(ChannelMeta, meta)


M = TypeVar("M", bound=ChannelMeta)


class ScopedContext:
    content: Dict[str, Any]

    def __init__(self, **kwargs) -> None:
        object.__setattr__(self, "content", kwargs)

    def __setattr__(self, __name: str, __value: Any) -> None:
        if __name == "content":
            raise AttributeError("'ScopedContext' object attribute 'content' is not overwritable")
        self.content[__name] = __value

    def __getattr__(self, __name: str) -> Any:
        if __name == "content":
            return self.content
        if __name not in self.content:
            raise AttributeError(f"{__name} is not defined")
        return self.content[__name]


class Channel(Generic[M]):
    module: str

    meta: M

    _export: Any = None
    _py_module: Optional[ModuleType] = None

    content: List[Cube]

    scopes: Dict[Type, ScopedContext]

    # TODO: _export reload for other modules

    def __init__(self, module: str) -> None:
        self.module = module
        self.meta = cast(M, _default_channel_meta())
        self.content = []
        self.scopes = {}

    @staticmethod
    def current() -> "Channel":
        """获取当前的 Channel 对象

        Returns:
            Channel: 当前的 Channel 对象
        """
        return channel_instance.get()

    def export(self, target):
        self._export = target
        return target

    def use(self, schema: BaseSchema):
        def use_wrapper(target: Union[Type, Callable, Any]):
            self.content.append(Cube(target, schema))
            return target

        return use_wrapper

    def cancel(self, target: Union[Type, Callable, Any]):
        self.content = [i for i in self.content if i.content is not target]

    def scoped_context(self, isolate_class: Type[Any]):
        context = self.scopes.setdefault(
            isolate_class,
            ScopedContext(channel=self),
        )
        contents = {id(i.content): i for i in self.content}
        members = inspect.getmembers(isolate_class, lambda obj: callable(obj) and id(obj) in contents)
        # 用 id 的原因: 防止一些 unhashable 的对象给我塞进来.
        for _, obj in members:
            contents[id(obj)].content = functools.partial(obj, context)
        return isolate_class



from typing import Any

from graia.broadcast import Broadcast

from graia.saya.behaviour import Behaviour
from graia.saya.cube import Cube

from .schema import ListenerSchema


class BroadcastBehaviour(Behaviour):
    broadcast: Broadcast

    def __init__(self, broadcast: Broadcast) -> None:
        self.broadcast = broadcast

    def allocate(
        self,
        cube: Cube[ListenerSchema],
    ):
        if isinstance(cube.metaclass, ListenerSchema):
            listener = cube.metaclass.build_listener(cube.content, self.broadcast)
            if not listener.namespace:
                listener.namespace = self.broadcast.getDefaultNamespace()
            self.broadcast.listeners.append(listener)
        else:
            return
        return True

    def release(self, cube: Cube) -> Any:
        if isinstance(cube.metaclass, ListenerSchema):
            self.broadcast.removeListener(self.broadcast.getListener(cube.content))
        else:
            return

        return True



"""Saya 相关的工具"""
from __future__ import annotations

import inspect
from typing import (
    Any,
    Callable,
    Dict,
    Generator,
    List,
    Type,
    TypeVar,
    Union,
    overload,
)

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.typing import T_Dispatcher
from graia.saya.factory import BufferModifier, SchemaWrapper, buffer_modifier, factory

from .schema import ListenerSchema

T_Callable = TypeVar("T_Callable", bound=Callable)
Wrapper = Callable[[T_Callable], T_Callable]

T = TypeVar("T")


def gen_subclass(cls: type[T]) -> Generator[type[T], Any, Any]:
    yield cls
    for sub in cls.__subclasses__():
        yield from gen_subclass(sub)


@buffer_modifier
def dispatch(*dispatcher: T_Dispatcher) -> BufferModifier:
    """附加参数解析器，最后必须接 `listen` 才能起效

    Args:
        *dispatcher (T_Dispatcher): 参数解析器

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """

    return lambda buffer: buffer.setdefault("dispatchers", []).extend(dispatcher)


@overload
def decorate(*decorator: Decorator) -> Wrapper:
    """附加多个无头装饰器

    Args:
        *decorator (Decorator): 无头装饰器

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """
    ...


@overload
def decorate(name: str, decorator: Decorator, /) -> Wrapper:
    """给指定参数名称附加装饰器

    Args:
        name (str): 参数名称
        decorator (Decorator): 装饰器

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """
    ...


@overload
def decorate(mapping: Dict[str, Decorator], /) -> Wrapper:
    """给指定参数名称附加装饰器

    Args:
        mapping (Dict[str, Decorator]): 参数名称与装饰器的映射

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """
    ...


@buffer_modifier
def decorate(*args) -> BufferModifier:
    """给指定参数名称附加装饰器

    Args:
        name (str | Dict[str, Decorator]): 参数名称或与装饰器的映射
        decorator (Decorator): 装饰器

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """
    arg: Union[Dict[str, Decorator], List[Decorator]]
    if isinstance(args[0], str):
        name: str = args[0]
        decorator: Decorator = args[1]
        arg = {name: decorator}
    elif isinstance(args[0], dict):
        arg = args[0]
    else:
        arg = list(args)

    def wrapper(buffer: Dict[str, Any]) -> None:
        if isinstance(arg, list):
            buffer.setdefault("decorators", []).extend(arg)
        elif isinstance(arg, dict):
            buffer.setdefault("decorator_map", {}).update(arg)

    return wrapper


@buffer_modifier
def priority(level: int, *events: Type[Dispatchable]) -> BufferModifier:
    """设置事件优先级

    Args:
        level (int): 事件优先级
        *events (Type[Dispatchable]): 提供时则会设置这些事件的优先级, 否则设置全局优先级

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """

    def wrapper(buffer: Dict[str, Any]) -> None:
        if events:
            buffer.setdefault("extra_priorities", {}).update((e, level) for e in events)
        else:
            buffer["priority"] = level

    return wrapper


@factory
def listen(*event: Union[Type[Dispatchable], str]) -> SchemaWrapper:
    """在当前 Saya Channel 中监听指定事件

    Args:
        *event (Union[Type[Dispatchable], str]): 事件类型或事件名称

    Returns:
        Callable[[T_Callable], T_Callable]: 装饰器
    """
    EVENTS: Dict[str, Type[Dispatchable]] = {e.__name__: e for e in gen_subclass(Dispatchable)}
    events: List[Type[Dispatchable]] = [e if isinstance(e, type) else EVENTS[e] for e in event]

    def wrapper(func: Callable, buffer: Dict[str, Any]) -> ListenerSchema:
        decorator_map: Dict[str, Decorator] = buffer.pop("decorator_map", {})
        buffer["inline_dispatchers"] = buffer.pop("dispatchers", [])
        if decorator_map:
            sig = inspect.signature(func)
            for param in sig.parameters.values():
                if decorator := decorator_map.get(param.name):
                    setattr(param, "_default", decorator)
            func.__signature__ = sig
        return ListenerSchema(listening_events=events, **buffer)

    return wrapper



from dataclasses import dataclass
from typing import Any, Generic, Optional, TypeVar

from .schema import BaseSchema

T = TypeVar("T", bound=Optional[BaseSchema])


@dataclass(init=True)
class Cube(Generic[T]):
    content: Any
    metaclass: T



from __future__ import annotations

from contextlib import suppress
from typing import TYPE_CHECKING

from creart import AbstractCreator, CreateTargetInfo, exists_module, it

from . import Saya

if TYPE_CHECKING:
    from .builtins.broadcast import BroadcastBehaviour


class SayaCreator(AbstractCreator):
    targets = (
        CreateTargetInfo(
            module="graia.saya",
            identify="Saya",
            humanized_name="Saya",
            description="<common,graia,saya> a modular implementation with modern design and injection",
            author=["GraiaProject@github"],
        ),
    )

    @staticmethod
    def create(create_type: type[Saya]) -> Saya:
        try:
            from graia.broadcast import Broadcast

            from .builtins.broadcast import BroadcastBehaviour

            broadcast = it(Broadcast)
            saya = create_type(broadcast)
            saya.install_behaviours(it(BroadcastBehaviour))
        except (ImportError, TypeError):
            saya = create_type()

        with suppress(ImportError, TypeError):
            from graia.scheduler.saya.behaviour import GraiaSchedulerBehaviour

            saya.install_behaviours(it(GraiaSchedulerBehaviour))

        return saya


class BroadcastBehaviourCreator(AbstractCreator):
    targets = (
        CreateTargetInfo(
            "graia.saya.builtins.broadcast.behaviour", "BroadcastBehaviour"
        ),
    )

    @staticmethod
    def available() -> bool:
        return exists_module("graia.broadcast")

    @staticmethod
    def create(create_type: type[BroadcastBehaviour]) -> BroadcastBehaviour:
        from graia.broadcast import Broadcast

        broadcast = it(Broadcast)
        return create_type(broadcast)



from typing import TYPE_CHECKING

from .context import channel_instance

if TYPE_CHECKING:
    from .channel import Channel


class BaseSchema:
    channel: "Channel"

    def __post_init__(self):
        self.channel = channel_instance.get()



from abc import ABCMeta, abstractmethod
from typing import Any

from graia.saya.cube import Cube


class Behaviour(metaclass=ABCMeta):
    @abstractmethod
    def allocate(self, cube: Cube[Any]) -> Any:
        pass

    @abstractmethod
    def release(self, cube: Cube[Any]) -> Any:
        pass



import functools
from typing import Any, Callable, Dict, TypeVar

from typing_extensions import ParamSpec

from .channel import Channel
from .cube import Cube
from .schema import BaseSchema


def ensure_buffer(func: Callable) -> Dict[str, Any]:
    if not hasattr(func, "__schema_buffer__"):
        setattr(func, "__schema_buffer__", {})
    return getattr(func, "__schema_buffer__")


P = ParamSpec("P")
T_Callable = TypeVar("T_Callable", bound=Callable)
SchemaWrapper = Callable[[Callable, Dict[str, Any]], BaseSchema]
BufferModifier = Callable[[Dict[str, Any]], None]
Wrapper = Callable[[T_Callable], T_Callable]


def factory(func: Callable[P, SchemaWrapper]) -> Callable[P, Wrapper]:
    @functools.wraps(func)
    def wrapper(*args: P.args, **kwargs: P.kwargs) -> Wrapper:
        wrapper: SchemaWrapper = func(*args, **kwargs)

        def register(func: T_Callable) -> T_Callable:
            schema: BaseSchema = wrapper(func, ensure_buffer(func))
            Channel.current().content.append(Cube(func, schema))
            return func

        return register

    return wrapper


def buffer_modifier(func: Callable[P, BufferModifier]) -> Callable[P, Wrapper]:
    @functools.wraps(func)
    def wrapper(*args: P.args, **kwargs: P.kwargs) -> Wrapper:
        modifier: BufferModifier = func(*args, **kwargs)

        def modify(func: T_Callable) -> T_Callable:
            modifier(ensure_buffer(func))
            return func

        return modify

    return wrapper



from dataclasses import dataclass
from typing import TYPE_CHECKING, List

if TYPE_CHECKING:
    from graia.saya.cube import Cube

    from .entity import Behaviour


@dataclass(init=True)
class RequireContext:
    module: str
    behaviours: List["Behaviour"]
    _index: int = 0


@dataclass(init=True)
class AllocationContext:
    cube: "Cube"


@dataclass(init=True)
class RouteContext:
    module: str



from dataclasses import dataclass, field
from typing import Callable, Dict, List, Optional, Type

from graia.broadcast import Broadcast
from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.entities.listener import Listener
from graia.broadcast.entities.namespace import Namespace
from graia.broadcast.typing import T_Dispatcher
from graia.saya.schema import BaseSchema


@dataclass
class ListenerSchema(BaseSchema):
    listening_events: List[Type[Dispatchable]]
    namespace: Optional[Namespace] = None
    inline_dispatchers: List[T_Dispatcher] = field(default_factory=list)
    decorators: List[Decorator] = field(default_factory=list)
    priority: int = 16
    extra_priorities: Dict[Type[Dispatchable], int] = field(default_factory=dict)

    def build_listener(self, callable: Callable, broadcast: "Broadcast"):
        listener = Listener(
            callable=callable,
            namespace=self.namespace or broadcast.getDefaultNamespace(),
            listening_events=self.listening_events,
            inline_dispatchers=self.inline_dispatchers,
            decorators=self.decorators,
            priority=self.priority,
        )
        if hasattr(listener, "priorities"):  # backward compatibility
            listener.priorities.update(self.extra_priorities.items())  # for type checkers
        return listener



from .context import AllocationContext as AllocationContext
from .context import RequireContext as RequireContext
from .entity import Behaviour as Behaviour
from .interface import BehaviourInterface as BehaviourInterface



from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.interfaces.dispatcher import DispatcherInterface

from graia.saya.channel import Channel
from graia.saya.context import saya_instance


class SayaModuleInstalled(Dispatchable):
    """不用返回 RemoveMe, 因为 Cube 在被 uninstall 时会被清理掉, 如果你用的是规范的 Saya Protocol 的话."""

    module: str
    channel: Channel

    def __init__(self, module: str, channel: Channel) -> None:
        self.module = module
        self.channel = channel

    class Dispatcher(BaseDispatcher):
        @staticmethod
        async def catch(interface: "DispatcherInterface[SayaModuleInstalled]"):
            from graia.saya import Saya

            if interface.annotation is Saya:
                return saya_instance.get()
            elif interface.annotation is Channel:
                return interface.event.channel


class SayaModuleUninstall(Dispatchable):
    """不用返回 RemoveMe, 因为 Cube 在被 uninstall 时会被清理掉, 如果你用的是规范的 Saya Protocol 的话."""

    module: str
    channel: Channel

    def __init__(self, module: str, channel: Channel) -> None:
        self.module = module
        self.channel = channel

    class Dispatcher(BaseDispatcher):
        @staticmethod
        async def catch(interface: "DispatcherInterface[SayaModuleUninstall]"):
            from graia.saya import Saya

            if interface.annotation is Saya:
                return saya_instance.get()
            elif interface.annotation is Channel:
                return interface.event.channel


class SayaModuleUninstalled(Dispatchable):
    """不用返回 RemoveMe, 因为 Cube 在被 uninstall 时会被清理掉, 如果你用的是规范的 Saya Protocol 的话."""

    module: str

    def __init__(self, module: str) -> None:
        self.module = module

    class Dispatcher(BaseDispatcher):
        @staticmethod
        async def catch(interface: "DispatcherInterface[SayaModuleUninstalled]"):
            from graia.saya import Saya

            if interface.annotation is Saya:
                return saya_instance.get()



import itertools
from typing import TYPE_CHECKING, Any, Generator, List, Optional

from graia.broadcast.exceptions import RequirementCrashed

from graia.saya.cube import Cube

from .context import AllocationContext, RequireContext
from .entity import Behaviour

if TYPE_CHECKING:
    from graia.saya import Saya

# TODO: Lifecycle for Behaviour


class BehaviourInterface:
    saya: "Saya"

    require_contents: List[RequireContext]

    def __init__(self, saya_instance: "Saya") -> None:
        self.saya = saya_instance
        self.require_contents = [RequireContext("graia.saya.__special__.global_behaviours", [])]

    @property
    def currentModule(self):
        return self.require_contents[-1].module

    @property
    def _index(self):
        return self.require_contents[-1]._index

    def require_context(self, module: str, behaviours: Optional[List["Behaviour"]] = None):
        self.require_contents.append(RequireContext(module, behaviours or []))
        return self

    def __enter__(self) -> "BehaviourInterface":
        return self

    def __exit__(self, _, exc: Exception, tb):
        self.require_contents.pop()  # just simple.
        if tb is not None:
            raise exc.with_traceback(tb)

    def behaviour_generator(self):
        yield from self.require_contents[0].behaviours
        # Cube 没有 behaviours 设定, 哦, 连 always 都没有.
        yield from self.require_contents[-1].behaviours

    def allocate_cube(self, cube: Cube) -> Any:
        start_offset = self._index + int(bool(self._index))

        for self.require_contents[-1]._index, behaviour in enumerate(
            itertools.islice(self.behaviour_generator(), start_offset, None, None),
            start=start_offset,
        ):
            result = behaviour.allocate(cube)

            if result is None:
                continue

            self.require_contents[-1]._index = 0
            return result
        else:
            raise RequirementCrashed(f"the dispatching requirement crashed: {cube}")

    def release_cube(self, cube: Cube) -> Any:
        start_offset = self._index + int(bool(self._index))

        for self.require_contents[-1]._index, behaviour in enumerate(
            itertools.islice(self.behaviour_generator(), start_offset, None, None),
            start=start_offset,
        ):
            result = behaviour.release(cube)

            if result is None:
                continue

            self.require_contents[-1]._index = 0
            return result
        else:
            raise RequirementCrashed(f"the dispatching requirement crashed: {cube}")



from __future__ import annotations

import importlib
import sys
from contextlib import contextmanager
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union

from loguru import logger

from graia.saya.behaviour import Behaviour, BehaviourInterface
from graia.saya.channel import Channel

from typing_extensions import deprecated

from .context import channel_instance, environment_metadata, saya_instance

if TYPE_CHECKING:
    from graia.broadcast import Broadcast


class Saya:
    """Modular application for Graia Framework.

    > 名称取自作品 魔女之旅 中的角色 "沙耶(Saya)", 愿所有人的心中都有一位活泼可爱的炭之魔女.

    Saya 的架构分为: `Saya Controller`(控制器), `Module Channel`(模块容器), `Cube`(内容容器), `Behaviour`(行为).

     - `Saya Controller` 负责管理各个模块的引入, 即本 `Saya` 类
     - `Module Channel` 负责在模块内收集组件信息
     - `Cube` 则用于使用 `Schema` 构造的元信息实例去描述内容
     - `Behaviour` 用于通过对 `Cube` 内的元信息进行解析, 并提供对其他已有接口的 Native 支持
    """

    behaviour_interface: BehaviourInterface
    behaviours: List[Behaviour]
    channels: Dict[str, Channel]
    broadcast: Optional[Broadcast]

    mounts: Dict[str, Any]

    def __init__(self, broadcast: Optional[Broadcast] = None) -> None:
        self.channels = {}
        self.behaviours = []
        self.behaviour_interface = BehaviourInterface(self)
        self.behaviour_interface.require_contents[0].behaviours = self.behaviours

        self.mounts = {}
        self.broadcast = broadcast

    @contextmanager
    def module_context(self):
        saya_token = saya_instance.set(self)
        yield
        saya_instance.reset(saya_token)

    @staticmethod
    def current() -> "Saya":
        """返回当前上下文中的 Saya 实例

        Returns:
            Saya: 当前上下文中的 Saya 实例
        """
        return saya_instance.get()

    def require_resolve(self, module: str) -> Channel:
        """导入模块, 并注入 Channel, 模块完成内容注册后返回该 Channel

        Args:
            module (str): 需为可被当前运行时访问的 Python Module 的引入路径

        Returns:
            Channel: 已注册了内容的 Channel.
        """
        channel = Channel(module)
        channel_token = channel_instance.set(channel)

        try:
            imported_module = importlib.import_module(module, module)
            channel._py_module = imported_module
            with self.behaviour_interface.require_context(module) as interface:
                for cube in channel.content:
                    try:
                        interface.allocate_cube(cube)
                    except:
                        logger.exception(f"an error occurred while loading the module's cube: {module}::{cube}")
                        raise
        finally:
            channel_instance.reset(channel_token)

        return channel

    @staticmethod
    def current_env() -> Any:
        """只能用于模块内. 返回在调用 Saya.require 方法时传入的 `require_env` 参数的值

        Returns:
            Any: 在调用 Saya.require 方法时传入的 `require_env` 参数的值
        """
        return environment_metadata.get(None)

    def require(self, module: str, require_env: Any = None) -> Union[Channel, Any]:
        """既处理 Channel, 也处理 Channel 中的 export 返回

        Args:
            module (str): 需为可被当前运行时访问的 Python Module 的引入路径
            require_env (Any, optional): 可以把这个参数作为硬编码配置使用, 虽然应该是叫传入 "环境条件".

        Returns:
            Union[Channel, Any]: 大多数情况下应该是 Channel, 如果设定了 export 就不一样了
        """
        logger.debug(f"require {module}")

        if module in self.channels:
            channel = self.channels[module]
            if channel._export:
                return channel._export
            return channel

        env_token = environment_metadata.set(require_env)
        channel = self.require_resolve(module)
        self.channels[module] = channel
        environment_metadata.reset(env_token)

        if self.broadcast:
            from .event import SayaModuleInstalled

            token = saya_instance.set(self)
            self.broadcast.postEvent(
                SayaModuleInstalled(
                    module=module,
                    channel=channel,
                )
            )
            saya_instance.reset(token)

        logger.info(f"module loading finished: {module}")

        if channel._export:
            return channel._export

        return channel

    def install_behaviours(self, *behaviours: Behaviour):
        """在控制器中注册 Behaviour, 用于处理模块提供的内容"""
        self.behaviours.extend(behaviours)

    def uninstall_channel(self, channel: Channel):
        """卸载指定的 Channel

        Args:
            channel (Channel): 需要卸载的 Channel

        Raises:
            TypeError: 提供的 Channel 不在本 Saya 实例内
            ValueError: 尝试卸载 __main__, 即主程序所属的模块
        """
        if channel not in self.channels.values():
            raise TypeError("assert an existed channel")

        if channel.module == "__main__":
            raise ValueError("main channel cannot uninstall")

        # TODO: builtin signal(async or sync)
        if self.broadcast:
            from .event import SayaModuleUninstall

            token = saya_instance.set(self)
            self.broadcast.postEvent(
                SayaModuleUninstall(
                    module=channel.module,
                    channel=channel,
                )
            )
            saya_instance.reset(token)

        with self.behaviour_interface.require_context(channel.module) as interface:
            for cube in channel.content:
                try:
                    interface.release_cube(cube)
                except:
                    logger.exception(f"an error occurred while loading the module's cube: {channel.module}::{cube}")
                    raise

        del self.channels[channel.module]
        channel._py_module = None

        if sys.modules.get(channel.module):
            del sys.modules[channel.module]

        if self.broadcast:
            from .event import SayaModuleUninstalled

            token = saya_instance.set(self)
            self.broadcast.postEvent(
                SayaModuleUninstalled(
                    module=channel.module,
                )
            )
            saya_instance.reset(token)

    def reload_channel(self, channel: Channel) -> None:
        """重载指定的模块

        Args:
            channel (Channel): 指定需要重载的模块, 请使用 channels.get 方法获取

        Raises:
            TypeError: 没有给定需要被重载的模块(`channel` 与 `module` 都不给定)
            ValueError: 没有通过 `module` 找到对应的 Channel
        """
        self.uninstall_channel(channel)
        new_channel: Channel = self.require_resolve(channel.module)

        channel.meta = new_channel.meta
        channel._export = new_channel._export
        channel._py_module = new_channel._py_module
        channel.content = new_channel.content

        self.channels[channel.module] = channel

    @deprecated("create_main_channel is deprecated, use main_context instead", category=DeprecationWarning)
    def create_main_channel(self) -> Channel:
        """创建不可被卸载的 `__main__` 主程序模块

        Returns:
            Channel: 属性 `name` 值为 `__main__`, 且无法被 `uninstall_channel` 卸载的模块.
        """
        may_current = self.channels.get("__main__")
        if may_current:
            return may_current

        main_channel = Channel("__main__")
        self.channels["__main__"] = main_channel

        if self.broadcast:
            from .event import SayaModuleInstalled

            token = saya_instance.set(self)
            self.broadcast.postEvent(
                SayaModuleInstalled(
                    module="__main__",
                    channel=main_channel,
                )
            )
            saya_instance.reset(token)

        return main_channel

    @contextmanager
    def main_context(self):
        """创建或加载不可被卸载的 `__main__` 主程序上下文, 如同使用 `Saya.require` 加载模块一样

        在单文件结构/临时测试环境下推荐使用该方法; 在正式项目中, 请使用 `Saya.require` 加载模块.

        Examples:
            ```python
            >>> with saya.main_context() as main_channel:
            >>>     # do something
            ```
        """
        if "__main__" not in self.channels:
            main_channel = Channel("__main__")
            self.channels["__main__"] = main_channel
        else:
            main_channel = self.channels["__main__"]
        token = channel_instance.set(main_channel)
        yield main_channel
        try:
            with self.behaviour_interface.require_context("__main__") as interface:
                for cube in main_channel.content:
                    try:
                        interface.allocate_cube(cube)
                    except:
                        logger.exception(f"an error occurred while loading the module's cube: __main__::{cube}")
                        raise
        finally:
            channel_instance.reset(token)

        if self.broadcast:
            from .event import SayaModuleInstalled

            token = saya_instance.set(self)
            self.broadcast.postEvent(
                SayaModuleInstalled(
                    module="__main__",
                    channel=main_channel,
                )
            )
            saya_instance.reset(token)

    def mount(self, mount_point: str, target):
        """挂载实例到 Saya 下, 以便整个模块系统共用.

        Args:
            mount_point (str): 指定的挂载点, 建议使用类似 `saya.builtin.asyncio.event_loop` 这样的形式
            target (Any): 需要挂载的实例

        Returns:
            NoReturn: 已将实例挂载, 可能把已经注册的挂载点给覆盖了.
        """
        self.mounts[mount_point] = target

    def unmount(self, mount_point: str):
        """删除挂载及其挂载点

        Args:
            mount_point (str): 目标挂载点

        Returns:
            NoReturn: 已经删除

        Raises:
            KeyError: 挂载点不存在
        """
        del self.mounts[mount_point]

    def access(self, mount_point: str):
        """访问特定挂载点

        Args:
            mount_point (str): 目标挂载点

        Returns:
            Any: 已经挂载的实例

        Raises:
            KeyError: 挂载点不存在
        """
        return self.mounts[mount_point]
